import os
import time
import yaml
import google.auth.crypt
import google.auth.jwt
from itertools import product
from typing import Dict, Tuple, Union
from google.oauth2.service_account import Credentials
from google.auth.impersonated_credentials import Credentials as ImpersonatedCredentials
from google.auth.transport.requests import Request
from google.auth.iam import Signer

MAX_TOKEN_LIFETIME_SECS = 3600  # seconds
CONFIG_TYPE = Dict[str, Union[str, Dict[str, str]]]


def load_client_configuration(configuration_path: str) -> CONFIG_TYPE:
    if os.path.exists(configuration_path):
        if configuration_path.endswith(".yaml") or configuration_path.endswith(".yml") or \
                configuration_path.endswith(".json"):
            with open(configuration_path, "r") as stream:
                configuration = yaml.safe_load(stream)
            svc_config = configuration.get("service")
            auth = configuration.get("authentication")
            if svc_config is not None and auth is not None and "issuer" in auth and \
                    all(j in svc_config and k in svc_config[j]
                        for j, k in product(["production", "development"],
                                            ["host", "api_key", "service_account_credentials"])):
                return configuration
            raise KeyError("load_client_configuration: Configuration file is missing keys.")
        raise TypeError(f"load_client_configuration: The configuration file must be a yaml file or a json file "
                        f"with extensions '.yaml', '.yml', or '.json'.  The configuration path given was "
                        f"{configuration_path}")
    raise FileNotFoundError(f"load_configuration: The file {configuration_path} does not exist!")


def generate_jwt_token_from_impersonated_account(service_account_info: Dict[str, str], audiences: str, issuer: str) -> Tuple[str, int]:
    credentials = Credentials.from_service_account_info(service_account_info,
                                                        scopes=["https://www.googleapis.com/auth/cloud-platform",
                                                                "https://www.googleapis.com/auth/iam"])
    if not credentials.valid:
        credentials.refresh(Request())
    impersonated_credentials = ImpersonatedCredentials(source_credentials=credentials, target_principal=issuer,
                                                       target_scopes=["https://www.googleapis.com/auth/cloud-platform"])
    if not impersonated_credentials.valid:
        impersonated_credentials.refresh(Request())
    signer = Signer(Request(), impersonated_credentials, impersonated_credentials.service_account_email)
    now = int(time.time())
    expires = now + MAX_TOKEN_LIFETIME_SECS

    payload = {
        'iat': now,
        'exp': expires,
        'aud': audiences,
        'iss': issuer
    }
    return google.auth.jwt.encode(signer, payload).decode("utf-8"), expires



from __future__ import print_function
import os
import sys
import willowlabs
import subprocess
from setuptools import Command, find_packages, setup
from setuptools.command.build_py import build_py
from setuptools.command.install import install


def compile_grpc_protos():
    import grpc_tools.protoc
    proto_files = [os.path.join(directory, filename) for directory, _, files in os.walk("protos")
                   for filename in files if filename.endswith(".proto")]
    grpc_tools.protoc.main(["grpc_tools.protoc", "--proto_path=protos/", "--python_out=."] + proto_files)


class CompileGRPC(Command):
    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            import grpc_tools.protoc
        except ModuleNotFoundError:
            subprocess.call("pip install grpcio-tools==1.27.2", shell=True)
        proto_files = [os.path.join(directory, filename) for directory, _, files in os.walk("protos")
                       for filename in files if filename.endswith(".proto")]
        subprocess.call(f"python -m grpc_tools.protoc --proto_path=protos/ --python_out=. --grpc_python_out=. "
                        f"{' '.join(proto_files)}", shell=True)


class BuildPyCommand(build_py):
    def run(self):
        subprocess.call("pip install --upgrade pip setuptools wheel", shell=True)
        self.run_command("compile_grpc")
        super(BuildPyCommand, self).run()


class InstallCommand(install):
    def run(self):
        subprocess.call("pip install --upgrade pip setuptools", shell=True)
        self.run_command("compile_grpc")
        super(InstallCommand, self).run()


if sys.version_info < (3, 7):
    print("google-api-python-client requires python3 version >= 3.7.", file=sys.stderr)
    sys.exit(1)


install_requires = [
    "google-api-python-client>=1.7.11",
    "grpcio>=1.27.0",
    "grpcio-tools>=1.27.2",
    "PyYAML>=5.3"
]

with open("README.rst", "r") as fh:
    long_description = fh.read()

setup(
    cmdclass={"compile_grpc": CompileGRPC, "build_py": BuildPyCommand},
    name="willowlabs",
    version=willowlabs.__version__,
    author="Willow Labs AS",
    author_email="lars@willowlabs.ai",
    description="A Python package for accessing the Willow Labs APIs",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/willowlabs/willowlabs-apis-python",
    python_requires=">=3.7.0",
    install_requires=install_requires,
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Topic :: Internet :: WWW/HTTP"
    ]
)
